import time

time.sleep(2)

import machine
import uasyncio as asyncio

from hal.st7789 import St7789
from hal.xpt2046 import Xpt2046
from hal.si5131 import SI5351, STRENGTH_2MA
from gui.display_driver_utils import Display_Driver

import lvgl as lv
from gui.async_utils import Lv_Async
from gui.dear_lvgl import *



machine.freq( 200_000_000 )

lcd_baudrate = 24_000_000
tsc_baudrate = 1_000_000
#adc_frequency = 10_000_000

spi_sck = machine.Pin( 10, machine.Pin.OUT )
spi_mosi= machine.Pin( 11, machine.Pin.OUT )
spi_miso= machine.Pin( 12, machine.Pin.IN  )
spi_dc  = machine.Pin( 8, machine.Pin.OUT )

lcd_rst = machine.Pin( 15, machine.Pin.OUT )
lcd_bl  = machine.Pin( 13, machine.Pin.OUT )
lcd_cs  = machine.Pin( 9, machine.Pin.OUT )
tsc_cs  = machine.Pin( 16, machine.Pin.OUT )

#adc_sck = machine.Pin( 21 )
#adc_db  = machine.Pin( 0 ) # 0..7 pins are used
        
# Calibration values
print( "tsc()" )
#AX, BX = 0.18072, 355.963
#AY, BY =-0.12825,-10.6669
AX, BX = 0.17571,-29.9520
AY, BY =-0.12669, 251.321
tsc = Xpt2046( tsc_baudrate, tsc_cs, spi_sck, spi_mosi, spi_miso, ax=AX, bx=BX, ay=AY, by=BY )

# Init lcd last one to start lvgl with SPI LCD baudrate
print( "lcd()" )
lcd = St7789( lcd_baudrate, lcd_cs, spi_sck, spi_mosi, spi_miso, spi_dc, lcd_rst, lcd_bl )

ST77XX_MADCTL = 0x36
ST77XX_MADCTL_MY = 0x80
ST77XX_MADCTL_MX = 0x40
ST77XX_MADCTL_MV = 0x20
ST77XX_MADCTL_ML = 0x10
ST77XX_MADCTL_RGB = 0x00
lcd.write_register( ST77XX_MADCTL, bytearray([ST77XX_MADCTL_MY | ST77XX_MADCTL_MV]) )
#lcd.clear( 0x0000 )

print( "adc()" )
#adc = Adc08100( adc_frequency, adc_sck, adc_db )

print( "lvgl" )
display_driver = Display_Driver( 320, 240, lcd, tsc )



class Synth:
    def __init__( self, parent ):
        sda = machine.Pin( 6 )
        scl = machine.Pin( 7 )

        mult = 100
        frequency = 10_000_000
        
        self.si5351 = SI5351( data=sda, clock=scl )
        
        self.si5351.clock_0.drive_strength( STRENGTH_2MA )
        self.si5351.clock_1.drive_strength( STRENGTH_2MA )
        self.si5351.clock_2.drive_strength( STRENGTH_2MA )

        self.si5351.set_frequency( frequency, self.si5351.clock_0, self.si5351.pll_a, mult )
        self.si5351.set_frequency( frequency, self.si5351.clock_1, self.si5351.pll_a, mult )
        self.si5351.set_frequency( frequency, self.si5351.clock_2, self.si5351.pll_a, mult )

        self.si5351.set_phase( self.si5351.clock_0, self.si5351.pll_a, mult )
        self.si5351.set_phase( self.si5351.clock_1, self.si5351.pll_a, mult )
        self.si5351.set_phase( self.si5351.clock_2, self.si5351.pll_a, mult )

        self.mult0 = 100
        self.freq0 = 10_000_000
        self.phase0 = 0
        self.mult1 = 100
        self.freq1 = 10_000_000
        self.phase1 = 0
        self.mult2 = 100
        self.freq2 = 10_000_000
        self.phase2 = 0
        
        self.parent = parent
        self.context = []
        self.widgets = {}
        self.build_ui( self.parent )
        
        #self.widgets["#F0"].set_value( self.freq0 )
        #self.widgets["#P0"].set_value( self.phase0 )
        #self.widgets["#F1"].set_value( self.freq1 )
        #self.widgets["#P1"].set_value( self.phase1 )
        self.widgets["#M2"].set_value( self.mult2 )
        self.widgets["#F2"].set_value( self.freq2 )
        self.widgets["#P2"].set_value( self.phase2 )
        
    def update( self ):
        #self.freq0 = self.widgets["#F0"].get_value()
        #self.phase0 = self.widgets["#P0"].get_value()
        #self.freq1 = self.widgets["#F1"].get_value()
        #self.phase1 = self.widgets["#P1"].get_value()
        self.mult2 = self.widgets["#M2"].get_value()
        self.freq2 = self.widgets["#F2"].get_value()
        self.phase2 = self.widgets["#P2"].get_value()
        try:
            mult = 100
            #self.si5351.set_frequency( self.freq0, self.si5351.clock_0, self.si5351.pll_a, mult )
            #self.si5351.set_frequency( self.freq1, self.si5351.clock_1, self.si5351.pll_a, mult )
            self.si5351.set_frequency( self.freq2, self.si5351.clock_2, self.si5351.pll_a, self.mult2 )

            #self.si5351.set_phase( self.si5351.clock_0, self.si5351.pll_a, mult )
            #self.si5351.set_phase( self.si5351.clock_1, self.si5351.pll_a, mult )
            self.si5351.set_phase( self.si5351.clock_2, self.si5351.pll_a, self.mult2 )
            self.widgets["Status"].set_text( "Status Ok" )
        except Exception as e:
            print( "except", e )
            self.widgets["Status"].set_text( "Status Fail" )
    
    def cb_freq0_inc( self, evt ):
        self.widgets["#F0"].increment()
        self.update()
    
    def cb_freq0_dec( self, evt ):
        self.widgets["#F0"].decrement()
        self.update()
    
    def cb_phase0_inc( self, evt ):
        self.widgets["#P0"].increment()
        self.update()
    
    def cb_phase0_dec( self, evt ):
        self.widgets["#P0"].decrement()
        self.update()
    

    def cb_freq1_inc( self, evt ):
        self.widgets["#F1"].increment()
        self.update()
    
    def cb_freq1_dec( self, evt ):
        self.widgets["#F1"].decrement()
        self.update()
    
    def cb_phase1_inc( self, evt ):
        self.widgets["#P1"].increment()
        self.update()
    
    def cb_phase1_dec( self, evt ):
        self.widgets["#P1"].decrement()
        self.update()


    def cb_freq2_inc( self, evt ):
        self.widgets["#F2"].increment()
        self.update()
    
    def cb_freq2_dec( self, evt ):
        self.widgets["#F2"].decrement()
        self.update()
    
    def cb_phase2_inc( self, evt ):
        self.widgets["#P2"].increment()
        self.update()
    
    def cb_phase2_dec( self, evt ):
        self.widgets["#P2"].decrement()
        self.update()
    
    def cb_mult2_inc( self, evt ):
        self.widgets["#M2"].increment()
        self.update()
    
    def cb_mult2_dec( self, evt ):
        self.widgets["#M2"].decrement()
        self.update()
    
    def build_ui( self, parent ):
        self.context.append( parent )
        set_context( self.context )
        set_widgets( self.widgets )
        with Column():
            add_label( "Status", w=100 )
            
            """
            add_label( "Clock0", w=100 )
            with Row():
                add_label( "Freq#0", w=60 )
                add_button( "+#F0", h=40 ).add_event_cb( self.cb_freq0_inc, lv.EVENT.CLICKED, None )
                add_spinbox( "#F0", w=120, h=40 )
                add_button( "-#F0", h=40 ).add_event_cb( self.cb_freq0_dec, lv.EVENT.CLICKED, None )
                            
            with Row():
                add_label( "Phase#0", w=60 )
                add_button( "+#P0", h=40 ).add_event_cb( self.cb_phase0_inc, lv.EVENT.CLICKED, None )
                add_spinbox( "#P0", w=120, h=40 )
                add_button( "-#P0", h=40 ).add_event_cb( self.cb_phase0_dec, lv.EVENT.CLICKED, None )
            
            add_label( "Clock1", w=100 )
            with Row():
                add_label( "Freq#1", w=60 )
                add_button( "+#F1", h=40 ).add_event_cb( self.cb_freq1_inc, lv.EVENT.CLICKED, None )
                add_spinbox( "#F1", w=120, h=40 )
                add_button( "-#F1", h=40 ).add_event_cb( self.cb_freq1_dec, lv.EVENT.CLICKED, None )
                            
            with Row():
                add_label( "Phase#1", w=60 )
                add_button( "+#P1", h=40 ).add_event_cb( self.cb_phase1_inc, lv.EVENT.CLICKED, None )
                add_spinbox( "#P1", w=120, h=40 )
                add_button( "-#P1", h=40 ).add_event_cb( self.cb_phase1_dec, lv.EVENT.CLICKED, None )
            """
            add_label( "Clock2", w=100 )
            with Row():
                add_label( "Mult#2", w=60 )
                add_button( "+#M2", h=40 ).add_event_cb( self.cb_mult2_inc, lv.EVENT.CLICKED, None )
                add_spinbox( "#M2", w=120, h=40 )
                add_button( "-#M2", h=40 ).add_event_cb( self.cb_mult2_dec, lv.EVENT.CLICKED, None )
                self.widgets["+#M2"].add_event_cb( self.cb_mult2_inc, lv.EVENT.LONG_PRESSED_REPEAT, None )
                self.widgets["-#M2"].add_event_cb( self.cb_mult2_dec, lv.EVENT.LONG_PRESSED_REPEAT, None )
            
            with Row():
                add_label( "Freq#2", w=60 )
                add_button( "+#F2", h=40 ).add_event_cb( self.cb_freq2_inc, lv.EVENT.CLICKED, None )
                add_spinbox( "#F2", w=120, h=40 )
                add_button( "-#F2", h=40 ).add_event_cb( self.cb_freq2_dec, lv.EVENT.CLICKED, None )
                self.widgets["+#F2"].add_event_cb( self.cb_freq2_inc, lv.EVENT.LONG_PRESSED_REPEAT, None )
                self.widgets["-#F2"].add_event_cb( self.cb_freq2_dec, lv.EVENT.LONG_PRESSED_REPEAT, None )
                            
            with Row():
                add_label( "Phase#2", w=60 )
                add_button( "+#P2", h=40 ).add_event_cb( self.cb_phase2_inc, lv.EVENT.CLICKED, None )
                add_spinbox( "#P2", w=120, h=40 )
                add_button( "-#P2", h=40 ).add_event_cb( self.cb_phase2_dec, lv.EVENT.CLICKED, None )
                self.widgets["+#P2"].add_event_cb( self.cb_phase2_inc, lv.EVENT.LONG_PRESSED_REPEAT, None )
                self.widgets["-#P2"].add_event_cb( self.cb_phase2_dec, lv.EVENT.LONG_PRESSED_REPEAT, None )

    
    def cb_enable( self, evt ):
        print( "enable" )

    def cb_disable( self, evt ):
        print( "disable" )

scr = lv.obj()
synth = Synth( scr )
lv.scr_load( scr )

print( "run" )
lva = Lv_Async( refresh_rate=20 )
asyncio.Loop.run_forever()


